#!/usr/bin/env bash

# The md5sums of the original and reprinted ISOs
fm_original_md5sum='088f9c250fc07cd0064c46ef93308533'
fm_reprint_md5sum='41acf140394bf0e0254294039607e260'

# Required programs that must be found in $PATH
dependencies=('xdelta3' 'md5sum')

# Test for programs required by this script before running
declare -a missing_dependencies=()
for dep in "${dependencies[@]}"; do
    type -P "$dep" >/dev/null \
        || missing_dependencies=( ${missing_dependencies[@]} "$dep" )
done
[[ -n "${missing_dependencies[*]}" ]] && {
    printf '%s\n' "${c_w}missing dependencies: $(
        for (( x=0; x < ${#missing_dependencies[@]}; x++ )); do
            printf '%s' "${missing_dependencies[$x]}"
            (( (( x + 1 )) < ${#missing_dependencies[@]} )) && printf '%s' ', '
        done
    )" >&2
    exit 1
}

unset confirm
while [[ ! "$confirm" =~ (y|Y) ]]; do
    # Start
    unset iso
    while [[ ! -f "$iso" ]]; do
        clear
        printf '%s\n' 'Please enter the name of your Front Mission 2nd ISO image:'
        read iso
    done

    # Version
    unset v
    while [[ ! "$v" =~ (r|o|R|O) ]]; do
        clear
        printf '%s\n' 'Is this an ISO of an (o)riginal release or a (r)eprint?'
        read v
    done

    # Original | Mission History (Reprint)
    case "$v" in
        r|R)
            # Mission History (Reprint) -> Original
            converter=His2Ori.xdelta
            patch='Reprint to Original'
            md5sum="$fm_reprint_md5sum"
            ;;
        o|O)
            # Original -> Mission History (Reprint)
            converter=Ori2His.xdelta
            patch='Original to Reprint'
            md5sum="$fm_original_md5sum"
            ;;
    esac

    # Exit with an error if the md5sum of the selected ISO doesn't match the one expected
    iso_md5sum=$(md5sum "$iso")
    [[ "${iso_md5sum/ *}" = "$md5sum" ]] || {
        printf '%s\n' "ERROR: The md5sum of '$iso' (${iso_md5sum/ *}) does not match the one expected for the ${patch/ *} ISO ($md5sum)" >&2
        exit 1
    }

    # Fail with an error if the required xdelta isn't in the current directory
    [[ -f "$converter" ]] || {
        printf '%s\n' "ERROR: The required file '$converter' could not be found" >&2
        exit 1
    }

    # Output
    unset write
    while [[ -z "$write" ]]; do
        clear
        printf '%s\n' 'Please enter a desired name for the patched file:'
        read write
    done

    # Confirm
    while [[ ! "$confirm" =~ (y|n|Y|N) ]]; do
        clear
        printf '%s\n' "ISO name: $iso"
        printf '%s\n' "Patch: $patch"
        printf '%s\n' "Output: $write"
        printf '%s\n' 'Is this information correct?'
        printf '%s\n' '(Y)es or (N)o'
        read confirm
    done
done

# Patch
clear
printf '%s\n' 'Ready to create patched ISO? (Enter to continue, Ctrl-c to quit)'
read

# Patching
printf '%s\n' 'Permission Granted!'
printf '%s\n' 'Please take note of any evt. error messages.'
printf '%s\n' 'Starting patching process! Please wait...'
xdelta3 -d -s "$iso" "$converter" "$write"

